
/**
 * A non-empty list of strings.
 * 
 * @version 1
 */
public class NESL implements StringList
{
    // instance variables
    private String first;
    private StringList rest;

    /**
     * Constructor for objects of class NESL
     * @param first   the first string in the list
     * @param rest   the remaining StringList
     */
    public NESL(String firstArg, StringList restArg)
    {
        // initialise instance variables
        this.first = firstArg;
        this.rest = restArg;
    }
    
    /**
     * getter for first
     * @return the first String
     */
    public String getFirst ()
    {
        return this.first;
    }
    
    /**
     * getter for rest
     * @return all but the first string in the list
     */
    public StringList getRest ()
    {
        return this.rest;
    }
    
    public boolean isEmpty ()
    {
        return false;
    }

    public int size()
    {
        /*
         * this       NESL    new NESL("Jake", new NESL("Crispin", list0))
         * this.first String  "Jake"
         * this.rest  StringList   list1, i.e. new NESL("Crispin", list0)
         * this.rest.isEmpty() boolean  false
         * this.rest.size()    int      1
         * Right answer    int          2
         */
        return this.rest.size() + 1;
    }
    
    public boolean equals (Object other)
    {
        if (! (other instanceof NESL))
        {
            return false;
        }
        else
        {
            NESL otherNESL = (NESL)other;
            return this.getFirst().equals(otherNESL.getFirst()) &&
                   this.getRest().equals(otherNESL.getRest());
        }
    }

    public StringList append (StringList other)
    {
         /*
         * this       NESL    ["x","y"]
         * other      StringList ["a","b","c","d"]
         * this.first String  "x"
         * this.rest  StringList   ["y"]
         * this.rest.isEmpty() boolean  false
         * this.rest.size()    int      1
         * this.rest.append(other)      StringList   ["y","a","b","c","d"]
         * this.rest.append(this.rest)  StringList   ["y","y"]
         * other.append(other)          StringList   ["a","b","c","d","a","b","c","d"]
         * other.append(this.rest)      StringList   ["a","b","c","d","y"]
         * right answer                 StringList   ["x","y","a","b","c","d"]
         * new NESL(this.first, this.rest.append(other))   StringList   ["x","y","a","b","c","d"]
         */
        return new NESL(this.first, this.rest.append(other));
    }
    
    public StringList reverse ()
    {
         /*
         * this       NESL    ["a","b","c","d"]
         * this.first String  "a"
         * this.rest  StringList   ["b","c","d"]
         * this.rest.reverse() StringList ["d","c","b"]
         * right answer                 StringList   ["d","c","b","a"]
         * this.rest.reverse().append(new NESL(this.first, new ESL()))
         */
        return this.rest.reverse().append(new NESL(this.first, new ESL()));       
    }
    
    public StringList sort ()
    {
         /*
         * this       NESL    ["c","d","b","a"]
         * this.first String  "c"
         * this.rest  StringList   ["d","b","a"]
         * this.rest.sort() StringList ["a","b","d"]
         * right answer                 StringList   ["a","b","c","d"]
         * this.rest.sort().insertInOrder(this.first)
         */
        return this.rest.sort().insertInOrder (this.first);
    }

    public StringList insertInOrder (String toInsert)
    {
         /*
         * this       sorted NESL    ["Crispin", "Jake", "Mary"]
         * this.first String  "Crispin"
         * toInsert   String  "Bob"                                                             "Kevin"
         * this.first.compareTo(toInsert)     int   >0                                             <0
         * this.rest  StringList   ["Jake", "Mary"]
         * this.rest.insertInOrder (this.first)   ["Crispin", "Jake", "Mary"]
         * this.rest.insertInOrder (toInsert)     ["Bob", "Jake", "Mary"]                  ["Jake", "Kevin", "Mary"]
         * right answer                 StringList   ["Bob", "Crispin", "Jake", "Mary"]   ["Crispin", "Jake", "Kevin", "Mary"]
         * new NESL(toInsert, this)       ["Bob", "Crispin", "Jake", "Mary"]             ["Kevin", "Crispin", "Jake", "Mary"]
         * new NESL(this.first, this.rest.insertInOrder(toInsert))                       ["Crispin", "Jake", "Kevin", "Mary"]
         */
        if (this.first.compareTo(toInsert) > 0)
        {
            return new NESL(toInsert, this);
        }
        else
        {
            return new NESL(this.first, this.rest.insertInOrder(toInsert));
        }
    }
 }
